@@ -5,24 +5,23 @@ use serde::Deserialize;
 use serde_json::json;
 use std::collections::HashMap;
 use std::collections::HashSet;
+
+use std::env;
+use std::path::PathBuf;
 use std::sync::mpsc;
 use std::sync::mpsc::{Receiver, Sender};
 use std::thread;
 
 use std::time::{Duration, SystemTime};
 
+mod common;
 mod config;
 mod journal;
 mod shelly;
 
+use crate::common::Payment;
 use crate::config::{Config, Device, HostPort, Price};
-use crate::journal::JournalEntry;
-
-#[derive(Deserialize, Debug, Clone)]
-struct Payment {
-    watt_hours: f64,
-    txid: String,
-}
+use crate::journal::{JournalEntry, JournalReader, JournalWriter};
 
 #[derive(Deserialize, Debug, Clone)]
 struct MoneroResponse {
@@ -58,38 +57,76 @@ fn main() -> () {
                 .short('f')
                 .long("config")
                 .takes_value(true)
+                .default_value("/etc/cipo.toml")
                 .help("Config file"),
         )
+        .arg(
+            Arg::with_name("journal")
+                .short('j')
+                .long("journal")
+                .takes_value(true)
+                .help("Journal directory, default is ${CWD}/journal"),
+        )
         .get_matches();
 
     let config_file = matches.value_of("config").unwrap_or("/etc/cipo.toml");
+    let journal_dir = match matches.value_of("journal") {
+        Some(dir) => PathBuf::from(dir),
+        None => {
+            let current_dir = env::current_dir().unwrap();
+            let journal_dir = current_dir.join("journal");
+            journal_dir
+        }
+    };
+    let journal_dir2 = journal_dir.clone();
 
     info!("Cipo is starting up");
-    info!("Using config file {}", config_file);
+    info!("Config file: {}", config_file);
+    info!(
+        "Journal dir: {}",
+        String::from(journal_dir.to_string_lossy())
+    );
 
     let config: Config = config::load_from_file(&config_file.to_string());
 
     let (journal_tx, journal_rx): (Sender<JournalEntry>, Receiver<JournalEntry>) = mpsc::channel();
 
-    let (sender, receiver): (Sender<MoneroTransfer>, Receiver<MoneroTransfer>) = mpsc::channel();
+    let (monero_tx, monero_rx): (Sender<MoneroTransfer>, Receiver<MoneroTransfer>) =
+        mpsc::channel();
 
+    let (journal_reader_tx, journal_reader_rx): (Sender<Payment>, Receiver<Payment>) =
+        mpsc::channel();
+    let reader = JournalReader::new(journal_reader_tx, journal_dir.clone());
+
+    reader.read();
+
+    let journal = JournalWriter::new(journal_rx, journal_dir);
     thread::spawn(move || {
-        listen_for_monero_payments(sender, config.monero_rpc);
-    });
-    thread::spawn(move || {
-        journal::journal_writer(journal_rx);
+        journal.start();
     });
 
-    route_payments(receiver, journal_tx, config.device, &config.price);
+    thread::spawn(move || {
+        listen_for_monero_payments(monero_tx, config.monero_rpc);
+    });
+
+    route_payments(
+        monero_rx,
+        journal_tx,
+        config.device,
+        &config.price,
+        journal_reader_rx,
+    );
 }
 
 fn route_payments(
-    receiver: Receiver<MoneroTransfer>,
+    monero_rx: Receiver<MoneroTransfer>,
     journal: Sender<JournalEntry>,
     devices: Vec<Device>,
     price: &Price,
+    journal_reader: Receiver<Payment>,
 ) {
     let mut router = HashMap::new();
+    let mut processed_transactions: HashSet<String> = HashSet::new();
 
     for device in devices {
         let (sender2, receiver2): (Sender<Payment>, Receiver<Payment>) = mpsc::channel();
@@ -101,11 +138,25 @@ fn route_payments(
         });
     }
 
-    loop {
-        let transfer: MoneroTransfer = receiver.recv().unwrap();
+    for credit in journal_reader.try_iter() {
+        if processed_transactions.contains(&credit.txid) {
+            continue;
+        }
 
-        if journal::have_been_journaled(&transfer.txid) {
-            //already delivered electricity
+        match router.get(&credit.address) {
+            Some(channel) => {
+                if credit.watt_hours > 0.0 {
+                    info!("Got credit of {} for {} Wh", credit.txid, credit.watt_hours);
+                    channel.send(credit.clone());
+                }
+                processed_transactions.insert(credit.txid.clone());
+            }
+            None => error!("missing device for address"),
+        }
+    }
+
+    for transfer in monero_rx {
+        if processed_transactions.contains(&transfer.txid) {
             continue;
         }
 
@@ -113,10 +164,12 @@ fn route_payments(
             Some(channel) => {
                 let payment = Payment {
                     watt_hours: calculate_watt_hours(price.xmr_per_kwh, transfer.amount),
+                    address: transfer.address.clone(),
                     txid: transfer.txid.clone(),
                 };
 
                 channel.send(payment);
+                processed_transactions.insert(transfer.txid.clone());
             }
             None => error!("missing device for address {}", &transfer.address),
         }
@@ -124,7 +177,7 @@ fn route_payments(
 }
 
 fn calculate_watt_hours(xmr_per_kwh: f64, picomonero: u64) -> f64 {
-    let xmr: f64 = picomonero as f64 / 1000000000000.0;
+    let xmr: f64 = picomonero as f64 / 1_000_000_000_000.0;
 
     (xmr / xmr_per_kwh) * 1000.0
 }
@@ -163,8 +216,7 @@ fn waiting_for_payment_per_device(
     journal: Sender<JournalEntry>,
     device: &Device,
 ) {
-    loop {
-        let payment: Payment = receiver.recv().unwrap();
+    for payment in receiver {
         deliver_electricity(journal.clone(), device, payment);
     }
 }
@@ -237,13 +289,14 @@ fn iterate_monero_transactions(
 fn deliver_electricity(
     journal: Sender<JournalEntry>,
     device: &Device,
-    paid: Payment,
+    payment: Payment,
 ) -> std::io::Result<()> {
     journal
         .send(JournalEntry {
             time: SystemTime::now(),
-            txid: paid.txid.clone(),
-            remaining_watt_hours: paid.watt_hours,
+            address: payment.address.clone(),
+            txid: payment.txid.clone(),
+            remaining_watt_hours: payment.watt_hours,
         })
         .unwrap();
 
@@ -258,21 +311,23 @@ fn deliver_electricity(
                     None => {
                         start = Some(total);
                         info!("{}: Turing on, meter at {:.2} Wh", device.location, total);
+                        debug!("{}: Current txid: {}", device.location, payment.txid);
                         shelly::on(device);
                     }
                     Some(start) => {
-                        let end = start + paid.watt_hours;
+                        let end = start + payment.watt_hours;
 
                         journal
                             .send(JournalEntry {
                                 time: SystemTime::now(),
-                                txid: paid.txid.clone(),
+                                address: payment.address.clone(),
+                                txid: payment.txid.clone(),
                                 remaining_watt_hours: end - total,
                             })
                             .unwrap();
 
                         debug!(
-                            "{}: Load {:.1} W, meter at {:.3} Wh, will end at {:.3} Wh",
+                            "{}: Current load {:.1} W, meter at {:.3} Wh, will end at {:.3} Wh",
                             device.location, s.apower, total, end
                         );
 
@@ -280,6 +335,7 @@ fn deliver_electricity(
                             shelly::on(device);
                         } else {
                             info!("{}: Turing off, meter at {:.2} Wh", device.location, total);
+                            debug!("{}: Current txid: {}", device.location, payment.txid);
                             shelly::off(device);
 
                             return Ok(());
